/**
 * Reads doubles from a text file and stores them in an array.
 * Also prints them to the screen.
 *
 * Written by Ed Hong UWT Jan. 15, 2003
 *  Modified by Ed Hong, Sep. 29, 2003
 * 
 * @author edhong
 * @version 0.1
 */

import	java.io.*;
import  java.util.*;
import  java.text.*;

public class ArrayIOUtils {

   /**
    * Tests the ArrayIOUtils methods.
    * <p>
    * This method would not normally be invoked by users of the
    * ArrayIOUtils class.
    */
	public static
	void	main(String[] argv)
		{ // load and print an array of doubles; for debugging
		int	n;
		double [] newArray;

		if (argv.length == 0)
			{
				newArray = LoadDoubleArray();
			}
		else 	newArray = LoadDoubleArray(argv[0]);

		printDoubleArray(newArray);
		writeDoubleArray(newArray);
		return;
		}

			/**
		     * Loads data from a text file specified by user input.
		     * file should contain doubles separated by spaces.
		     *
		     */
	public static double [] LoadDoubleArray() {

		// ask user for directory and path name.
          System.out.print("Please enter the full path and file name for the input data: ");
		  String userinput;
		  userinput = KeyboardReader.readString();
   		  return    LoadDoubleArray(userinput);
	}

			/**
		     * Loads data from a text file specified by user input.
		     * file should contain integers separated by spaces.
		     *
		     */
	public static int [] LoadIntArray() {

		// ask user for directory and path name.
          System.out.print("Please enter the full path and file name for the input data: ");
		  String userinput;
		  userinput = KeyboardReader.readString();
   		  return    LoadIntArray(userinput);
	}

			/**
		     * Loads data  from a text file.
		     * file should contain doubles all on one line, separated by spaces.
		     * @param pathandfilename  is the name of the file, including full path.
		     */
	public static double [] LoadDoubleArray(String pathandfilename) {
		double [] theData;
		BufferedReader	inbuf = InputLib.fopen(pathandfilename);
		System.out.println("Opened " + pathandfilename + " for input.");
		String	line = InputLib.getLine(inbuf); // get first line

		StringTokenizer	sTok = new StringTokenizer(line);
		int n = sTok.countTokens();
		int i;
		if (n <1) {
		    System.err.println("Error: no tokens found!");
		    System.exit(1);
		}
		theData = new double[n];
		for (i=0; i <  theData.length; i++)
			theData[i] = Double.parseDouble(sTok.nextToken());

		InputLib.fclose(inbuf);
		System.out.println("Successfully loaded " + n + " doubles. ");
		return theData;
	}


			/**
		     * Loads data  from a text file.
		     * file should contain integers all on one line, separated by spaces.
		     * @param pathandfilename  is the name of the file, including full path.
		     */
	public static int [] LoadIntArray(String pathandfilename) {
		int [] theData;
		BufferedReader	inbuf = InputLib.fopen(pathandfilename);
		System.out.println("Opened " + pathandfilename + " for input.");
		String	line = InputLib.getLine(inbuf); // get first line

		StringTokenizer	sTok = new StringTokenizer(line);
		int n = sTok.countTokens();
		int i;
		if (n <1) {
		    System.err.println("Error: no tokens found!");
		    System.exit(1);
		}
		theData = new int[n];
		for (i=0; i <  theData.length; i++)
			theData[i] = Integer.parseInt(sTok.nextToken());

		InputLib.fclose(inbuf);
		System.out.println("Successfully loaded " + n + " integers. ");
		return theData;
	}


			/**
		     * Prints double data to the screen.
		     * truncates decimals somewhat for reading.
		     * @param D  is the data to print out.
		     */
	public static void printDoubleArray(double []D) {
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(3);
		String s;

		int i,j;
		int n = D.length;
		for (i=0; i < n; i++) {
				s = nf.format(D[i]);
				while (s.length() < 7)
				  s = " " + s;
				System.out.print(s);
			System.out.println("");
		}
	}


			/**
		     * Prints integer data to the screen.
		     * @param D  is the data to print out.
		     */
	public static void printIntArray(int []D) {
		String s;

		int i,j;
		int n = D.length;
		for (i=0; i < n; i++) {
				s = Integer.toString(D[i]);
				while (s.length() < 7)
				  s = " " + s;
				System.out.print(s);
			System.out.println("");
		}
	}




		   /**
		    * Writes double data to a file
		    * file name is specified by user input
		    * @param D is the data to store to a file.
		    */
	public static void writeDoubleArray(double []D) {
		// ask user for directory and path name.
        System.out.print("Please enter the full path and file name for the data to be stored: ");
		String userinput;
		userinput = KeyboardReader.readString();
		writeDoubleArray(D,userinput);
	}

		   /**
		    * Writes double data to a file
		    * @param D is the data to store to a file.
		    * @param filename is the filename (including full path).
		    */
	public static void writeDoubleArray(double []D, String filename) {
		System.out.println("Writing file " + filename);
		try {
		 PrintWriter output = new  PrintWriter(new FileWriter(filename));
		 int i;
		 int n = D.length;
		 for (i=0; i < n; i++) {
			   output.print(D[i] + " " );
		 }
         output.close();
        } //try
        catch (IOException e)
            {
             System.err.println("Error opening file" +e);
                return;
        }

	}

}
